package org.foxarmy.chatProxy

import com.google.gson.Gson
import org.apache.kafka.clients.consumer.Consumer
import org.apache.kafka.clients.consumer.ConsumerRecord
import org.apache.kafka.clients.consumer.ConsumerRecords
import org.apache.kafka.clients.consumer.KafkaConsumer
import org.apache.kafka.clients.producer.KafkaProducer
import org.apache.kafka.clients.producer.Producer
import org.apache.kafka.clients.producer.ProducerRecord
import org.apache.kafka.common.serialization.StringDeserializer
import org.apache.kafka.common.serialization.StringSerializer
import org.bukkit.Bukkit
import org.bukkit.entity.Player
import org.bukkit.event.player.AsyncPlayerChatEvent
import org.bukkit.plugin.java.JavaPlugin
import java.time.Duration
import java.util.*

class KafkaClient(plugin: JavaPlugin, brokers: List<String>) {
    private val consumer: Consumer<String, String>
    private val producer: Producer<String, String>

    init {
        Thread.currentThread().contextClassLoader = null;
        val props = Properties()

        props["bootstrap.servers"] = brokers
        props["group.id"] = "chat-listener"
        props["key.deserializer"] = StringDeserializer::class.java.name
        props["value.deserializer"] = StringDeserializer::class.java.name
        props["key.serializer"] = StringSerializer::class.java.canonicalName
        props["value.serializer"] = StringSerializer::class.java.canonicalName

        producer = KafkaProducer<String, String>(props)
        consumer = KafkaConsumer<String, String>(props)

        consumer.subscribe(listOf("chatMessage"))
        listenForMessages(plugin)
    }

    private fun listenForMessages(plugin: JavaPlugin) {
        Bukkit.getScheduler().runTaskTimer(
            plugin, Runnable {
                val records: ConsumerRecords<String, String> = consumer.poll(Duration.ofMillis(100))
                for (record: ConsumerRecord<String, String> in records) {
                    val recordJSON = Gson().fromJson(record.value(), MessageJson::class.java)
                    if (recordJSON.origin == "server") continue
                    Bukkit.broadcastMessage("<${recordJSON.author}> ${recordJSON.content}")
                    recordJSON.origin = "server"
                    sendData(recordJSON.toString())
                }
            }, 0L, 20L
        )
    }

    fun sendData(data: String) {
        val futureResult = producer.send(ProducerRecord<String, String>("chatMessage", data))
    }
}